from __future__ import annotations

from linearmodels.asset_pricing import (
    LinearFactorModel,
    LinearFactorModelGMM,
    TradedFactorModel,
)
from linearmodels.iv import IV2SLS, IVGMM, IVGMMCUE, IVLIML
from linearmodels.panel import (
    BetweenOLS,
    FamaMacBeth,
    FirstDifferenceOLS,
    PanelOLS,
    PooledOLS,
    RandomEffects,
)
from linearmodels.system import IV3SLS, SUR, IVSystemGMM

__all__ = [
    "traded_factor_model",
    "linear_factor_model",
    "linear_factor_model_gmm",
    "between_ols",
    "random_effects",
    "first_difference_ols",
    "pooled_ols",
    "panel_ols",
    "iv_2sls",
    "iv_gmm",
    "iv_gmm_cue",
    "iv_liml",
    "sur",
    "iv_3sls",
    "iv_system_gmm",
]


traded_factor_model = TradedFactorModel.from_formula
linear_factor_model = LinearFactorModel.from_formula
linear_factor_model_gmm = LinearFactorModelGMM.from_formula

iv_2sls = IV2SLS.from_formula
iv_liml = IVLIML.from_formula
iv_gmm = IVGMM.from_formula
iv_gmm_cue = IVGMMCUE.from_formula

panel_ols = PanelOLS.from_formula
pooled_ols = PooledOLS.from_formula
between_ols = BetweenOLS.from_formula
first_difference_ols = FirstDifferenceOLS.from_formula
random_effects = RandomEffects.from_formula
fama_macbeth = FamaMacBeth.from_formula

sur = SUR.from_formula
iv_3sls = IV3SLS.from_formula
iv_system_gmm = IVSystemGMM.from_formula
